@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2015, 2016 Trustees of Columbia University
+# Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,7 +36,11 @@
 #' @template args-sparse
 #' @template reference-gelman-hill
 #' 
-#' @param formula,data,family Same as for \code{\link[lme4]{glmer}}.
+#' @param formula,data,family Same as for \code{\link[lme4]{glmer}}. \emph{We
+#'   strongly advise against omitting the \code{data} argument}. Unless 
+#'   \code{data} is specified (and is a data frame) many post-estimation 
+#'   functions (including \code{update}, \code{loo}, \code{kfold}) are not 
+#'   guaranteed to work properly.
 #' @param subset,weights,offset Same as \code{\link[stats]{glm}}.
 #' @param na.action,contrasts Same as \code{\link[stats]{glm}}, but rarely 
 #'   specified.
@@ -88,6 +92,7 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
   
   call <- match.call(expand.dots = TRUE)
   mc <- match.call(expand.dots = FALSE)
+  data <- validate_data(data) #, if_missing = environment(formula))
   family <- validate_family(family)
   mc[[1]] <- quote(lme4::glFormula)
   mc$control <- make_glmerControl()
@@ -127,7 +132,8 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
   fit <- nlist(stanfit, family, formula, offset, weights, 
                x = if (getRversion() < "3.2.0") cBind(X, Z) else cbind2(X, Z), 
                y = y, data, call, terms = NULL, model = NULL, 
-               na.action, contrasts, algorithm, glmod)
+               na.action, contrasts, algorithm, glmod, 
+               modeling_function = "stan_glmer")
   out <- stanreg(fit)
   class(out) <- c(class(out), "lmerMod")
   
@@ -163,6 +169,7 @@ stan_lmer <- function(formula,
   mc$family <- "gaussian"
   out <- eval(mc, parent.frame())
   out$call <- call
+  out$modeling_function <- "stan_lmer"
   return(out)
 }
 
@@ -199,5 +206,6 @@ stan_glmer.nb <- function(formula,
   mc$family <- neg_binomial_2(link = link)
   out <- eval(mc, parent.frame())
   out$call <- call
+  out$modeling_function <- "stan_glmer.nb"
   return(out)
 }
