@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2015, 2016 Trustees of Columbia University
+# Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -360,8 +360,7 @@ test_that("draw_ystar_rng returns expected results", {
 
 # glmer
 context("glmer")
-test_that("the Stan equivalent of lme4's Z %*% b works", {
-  stopifnot(require(lme4))
+if (require(lme4) && require(HSAUR3)) test_that("the Stan equivalent of lme4's Z %*% b works", {
   stopifnot(require(Matrix))
   test_lme4 <- function(group) {
     Lambdati <- group$Lambdat
